<div class="alert alert-success d-flex align-items-center" role="alert">
    <svg xmlns="http://www.w3.org/2000/svg" width="24" height="24" fill="currentColor"
         class="bi bi-exclamation-triangle-fill flex-shrink-0 me-2" viewBox="0 0 16 16" role="img"
         aria-label="Warning:">
        <path d="M13 2.5a1.5 1.5 0 0 1 3 0v11a1.5 1.5 0 0 1-3 0zm-1 .724c-2.067.95-4.539 1.481-7 1.656v6.237a25 25 0 0 1 1.088.085c2.053.204 4.038.668 5.912 1.56zm-8 7.841V4.934c-.68.027-1.399.043-2.008.053A2.02 2.02 0 0 0 0 7v2c0 1.106.896 1.996 1.994 2.009l.496.008a64 64 0 0 1 1.51.048m1.39 1.081q.428.032.85.078l.253 1.69a1 1 0 0 1-.983 1.187h-.548a1 1 0 0 1-.916-.599l-1.314-2.48a66 66 0 0 1 1.692.064q.491.026.966.06"/>
    </svg>
    <div>
        The problem with PTMs has been resolved! Modified amino acids
        are displayed with an X in the sequence viewer. As AlphaFold outputs
        atom-wise PAE values for modified amino acids, they were replaced
        with their arithmetic mean to retrieve residue-wise values instead.
    </div>
    <button type="button" class="btn-close flex-shrink-0 align-self-start" style="margin-left: auto;"
            data-bs-dismiss="alert" aria-label="Close"></button>
</div>
