body {
    background-color: rgba(170, 170, 170, 0.4);
}

code {
    color: black;
}

.complex-sequence-viewer {
    margin-bottom: 20px;
}

.csv-container {
    max-height: 30ex;
}

.sc-options {
    display: none;
}

#split-view {
    display: flex;
    column-gap: 150px;
    justify-content: center;
    margin-bottom: 100px;
}

@media only screen and (max-width: 860px) {
    #split-view {
        flex-direction: column;
        align-items: center;
    }
}

#test-carousel {
    margin-top: 20px;
    width: 600px;
}

@supports (aspect-ratio: 1 / 1) {
    .carousel {
        aspect-ratio: 3 / 2;
    }
}

.sc-fullscreen-toggle {
    display: none;
}

.sv-caption {
    display: none;
}

#pae-viewer {
    margin-top: 120px;
    margin-bottom: 300px;
}

.pae-viewer {
    width: 360px;
    height: 360px;
}

#uploadStructure {
    margin-bottom: 50px;
}

#structure-panel {
    border-top: #eee solid 1px;
}

#help img {
    /*display: block;*/
    /*margin: auto;*/
}

.help-text {
    display: flex;
    flex-direction: row;
    justify-content: flex-start;
    align-items: center;
}

.help-columns {
    display: flex;
    flex-direction: row;
    justify-content: space-evenly;
    align-items: flex-start;
    text-align: left;
}

.toast {
    z-index: 100;
}

.header {
    background-color: white;
}
