.complex-sequence-viewer {
    box-shadow: 1px 1px 2px 1px rgba(0, 0, 0, 0.07);
}

.csv-container {
    max-height: 15ex;
    overflow-y: scroll;
}

.csv-sequence {
    display: flex;
    flex-wrap: wrap;
    font-family: "Courier New", "Courier", monospace;
    font-size: smaller;
    user-select: none;
    padding: 0.5em 1em 0.5em 1em;
}

.csv-segment {
    display: flex;
    flex-direction: column;
    justify-content: flex-end;
    width: 0.6em;
    height: 5ex;
}

.csv-label {
    position: sticky;
    top: 0;
    z-index: 10;
    background-color: lightgrey;
    font-weight: bold;
    font-style: italic;
    font-size: smaller;
    padding: 0.5ex 0.5em 0.5ex 0.5em;
}

.csv-index {
    font-size: x-small;
}

.csv-code {

}

.csv-code-highlighted {
    filter: brightness(150%);
    background-color: rgba(255, 255, 255, 0.5)
}

.csv-code-selecting {
    filter: brightness(150%);
    background-color: rgba(0, 255, 0, 0.5)
}

.csv-code-selected {
    filter: brightness(150%);
    background-color: rgba(255, 0, 255, 0.5)
}

.csv-code-selected-x {
    background-color: cyan;
}

.csv-code-selected-y {
    background-color: orange;
}

.csv-code-selected-overlap {
    background-color: magenta;
}

.csv-x {
    background-color: rgba(0, 255, 255, 0.2)
}

.csv-y {
    background-color: rgba(255, 165, 0, 0.2)
}

.csv-overlap {
    background-color: rgba(255, 0, 255, 0.2)
}


.csv-status-bar {
    display: flex;
    align-items: center;
    height: 3ex;
}

.csv-status-text {
    font-size: small;
    padding-left: 0.5em;
}
