<div class="tab-pane fade show active" id="examples" role="tabpanel"
     aria-labelledby="examples-tab">
  <form id="chooseExample">
    <fieldset id="chooseExampleFields" disabled>
    <!--
      <div class="mb-3">
        Choose one of the following examples or use the 'Upload' tab
        to upload your own protein complex data. The examples are
        complexes predicted by
        <a href="https://www.biorxiv.org/content/10.1101/2021.10.04.463034v2"
           target="_blank">AlphaFold-Multimer</a>
        from
        <a href="http://doi.org/10.15252/msb.202311544" target="_blank">
          O'Reilly <i>et al.</i> (2023)</a>.
      </div>

      <div>
        <button class="btn btn-outline-primary" name="id" value="161">
          PdhA-PdhB-PdhI
        </button>
        <button class="btn btn-outline-primary" name="id" value="160">
          SufD-SufC-SufB
        </button>
        <button class="btn btn-outline-primary" name="id" value="102">
          FliT-FliD
        </button>
      </div>
    -->
    </fieldset>
  </form>
</div>
