<div class="card">
  <div class="card-header">
    <h2>Help</h2>
  </div>
  <div class="card-body">
    <div class="accordion" id="help">
      {{paeViewerDemo__help__viewer.tpl}}
      {{paeViewerDemo__help__structure.tpl}}
      {{paeViewerDemo__help__sequence.tpl}}
      {{paeViewerDemo__help__ptms.tpl}}
    </div>
  </div>
</div>
