.structure-viewer {
    width: 100%;
    height: 100%;
}

/* 'sv-loading' from https://loading.io/css/*/
.sv-loading, .sv-error {
    width: inherit;
    height: inherit;
    position: absolute;
    z-index: 1;
    background-color: rgba(0, 0, 0, 50%);
    display: flex;
    align-items: center;
    justify-content: center;
}

.sv-loading div {
    box-sizing: border-box;
    display: block;
    position: absolute;
    width: 64px;
    height: 64px;
    margin: 8px;
    border: 2px solid;
    border-radius: 50%;
    animation: sv-loading 1.2s cubic-bezier(0.5, 0, 0.5, 1) infinite;
    border-color: #fff transparent transparent transparent;
}

.sv-loading div:nth-child(1) {
    animation-delay: -0.45s;
}

.sv-loading div:nth-child(2) {
    animation-delay: -0.3s;
}

.sv-loading div:nth-child(3) {
    animation-delay: -0.15s;
}

@keyframes sv-loading {
    0% {
        transform: rotate(0deg);
    }
    100% {
        transform: rotate(360deg);
    }
}

.sv-error > img {
    border: none;
    box-shadow: none;
    width: 20%;
    height: 20%;
    filter: invert(1);
}

.sv-error > p {
    position: absolute;
    color: white;
    top: 60%;
}

.sv-display {
    width: 100%;
    height: 100%;
}

.sv-caption {
    position: absolute;
    padding: 0;
    text-align: left;
    z-index: 1;
    left: var(--inset-sidebar);
    bottom: var(--inset-sidebar);
}

.sv-transparent-box {
    background-color: rgba(255, 255, 255, 90%);
    border-radius: 5px;
}

.sv-caption > a {
    color: var(--font-color);
    padding: 2px 4px 2px 4px;
}

.sv-screenshot {
    position: absolute;
    padding: 0;
    border-radius: 50%;
    z-index: 1;
    right: var(--inset-sidebar);
    top: var(--inset-sidebar);
    padding: 1%;
}

.sv-screenshot > input {
    display: block;
    filter: var(--svg-color-564b47);
}
