<div class="accordion-item">
  <h2 class="accordion-header" id="helpSequenceViewerHeading">
    <button class="accordion-button collapsed" type="button"
            data-bs-toggle="collapse" data-bs-target="#helpSequenceViewer"
            aria-expanded="true" aria-controls="helpSequenceViewer">
      <span class="fw-bold">Sequence Viewer</span>
    </button>
  </h2>
  <div id="helpSequenceViewer" class="accordion-collapse collapse"
       aria-labelledby="helpSequenceViewerHeading"
       data-bs-parent="#help">
    <div class="accordion-body">
      The sequence viewer shows all sequences of the complex subunits.
      When selecting a range of residues, the corresponding
      parts of the complex are highlighted in the structure viewer.
      The status bar shows gives information about the currently
      selected or highlighted sequence(s).
    </div>
  </div>
</div>
