<div class="accordion-item">
  <h2 class="accordion-header" id="helpStructureViewerHeading">
    <button class="accordion-button collapsed" type="button"
            data-bs-toggle="collapse" data-bs-target="#helpStructureViewer"
            aria-expanded="true" aria-controls="helpStructureViewer">
      <span class="fw-bold">Structure Viewer</span>
    </button>
  </h2>
  <div id="helpStructureViewer" class="accordion-collapse collapse"
       aria-labelledby="helpStructureViewerHeading"
       data-bs-parent="#help">
    <div class="accordion-body">
      The 3D structure viewer uses <a href="https://nglviewer.org/" target="_blank">
        NGL viewer
      </a> for molecular visualization.
      Hold the left mouse button and move the cursor to rotate the structure.
      Analogously, use the right mouse button to move it (alternatively,
      use the left mouse button while holding the Ctrl key).
      Use the wheel to zoom (alternatively, hold the left mouse button and
      move the cursor while holding the Shift key).
      Double-click on any part of the structure to focus onto it.
      You can rotate the structure around a fixed axis by holding the
      Ctrl key and pressing the left mouse button while moving the cursor.
    </div>
  </div>
</div>
